/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.library;

import static org.eclipse.emf.ecore.util.EcoreUtil.copyAll;

import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.IConnection;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.base.model.layout.ILayoutData;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.ILibrary;
import org.fortiss.tooling.kernel.model.ILibraryElement;
import org.fortiss.tooling.kernel.model.ILibraryElementReference;
import org.fortiss.tooling.kernel.model.ILibraryPackage;
import org.fortiss.tooling.kernel.model.INamedElement;
import org.fortiss.tooling.kernel.service.ILibraryService;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Higher level {@link ILibraryService} for {@link IModelElement} objects.
 * 
 * @author ratiu
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating GREEN Hash: E41A09449EB2BB933B660E2A6337545B
 */
public class ModelElementLibraryService implements ILibraryService {

	/** The instance of this service. */
	public static final ModelElementLibraryService INSTANCE = new ModelElementLibraryService();

	/** The private constructor. */
	private ModelElementLibraryService() {
		// do nothing
	}

	/** {@inheritDoc} */
	@Override
	public boolean isLibraryElementShadow(EObject anObject) {
		return ILibraryService.INSTANCE.isLibraryElementShadow(anObject);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isDirty(ILibraryElementReference libRef) {
		return ILibraryService.INSTANCE.isDirty(libRef);
	}

	/** {@inheritDoc} */
	@Override
	public void setDirty(ILibraryElementReference libRef, boolean flag) {
		ILibraryService.INSTANCE.setDirty(libRef, flag);
	}

	/** {@inheritDoc} */
	@Override
	public ILibraryElement findLibraryElementByURI(String uri) {
		return ILibraryService.INSTANCE.findLibraryElementByURI(uri);
	}

	/** {@inheritDoc} */
	@Override
	public List<ILibraryElement> getLibraryElementsFromAllWorkspaceLibraries() {
		return ILibraryService.INSTANCE.getLibraryElementsFromAllWorkspaceLibraries();
	}

	/** {@inheritDoc} */
	@Override
	public List<ILibrary> getLibrariesFromWorkspace() {
		return ILibraryService.INSTANCE.getLibrariesFromWorkspace();
	}

	/** {@inheritDoc} */
	@Override
	public List<ILibraryPackage> getAllLibraryPackages(ILibrary lib) {
		return ILibraryService.INSTANCE.getAllLibraryPackages(lib);
	}

	/** {@inheritDoc} */
	@Override
	public void ensureWrappedElementIsUpToDate(ILibraryElementReference ref) {
		ILibraryService.INSTANCE.ensureWrappedElementIsUpToDate(ref);
	}

	/** {@inheritDoc} */
	@Override
	public void addElementToLibrary(ILibraryPackage pack, INamedElement obj) {
		ILibraryService.INSTANCE.addElementToLibrary(pack, obj);
	}

	/**
	 * Converts a {@link ILibraryElementReference} into a {@link IHierarchicElement}.
	 */
	public void convertLibraryElementRefToHierarchicalElement(
			final ILibraryElementReference elementRef) {
		final IHierarchicElement wrapped = (IHierarchicElement)elementRef.getLibraryElementShadow();

		final ITopLevelElement topLevel =
				IPersistencyService.INSTANCE.getTopLevelElementFor(elementRef);
		topLevel.runAsCommand(new Runnable() {
			@Override
			public void run() {
				IHierarchicElement container = (IHierarchicElement)elementRef.eContainer();
				container.getContainedElementsList().remove(elementRef);

				// copy all connectors of the reference to the wrapped component
				EList<IConnector> wrappedConnectorsList = new BasicEList<IConnector>();
				wrappedConnectorsList.addAll(wrapped.getConnectorsList());
				wrapped.getConnectorsList().clear();
				EList<IConnector> elementRefConnectorsList = new BasicEList<IConnector>();
				elementRefConnectorsList.addAll(((IHierarchicElement)elementRef)
						.getConnectorsList());
				for(IConnector elemRefConnector : elementRefConnectorsList) {
					wrapped.getConnectorsList().add(elemRefConnector);
					String elementRefConnectorName = ((INamedElement)elemRefConnector).getName();

					// for each connector of the element reference add the inner connections from
					// the wrapped connector
					for(IConnector wrappedConnector : wrappedConnectorsList) {
						String currentWrappedConnectorName =
								((INamedElement)wrappedConnector).getName();

						if(currentWrappedConnectorName.equals(elementRefConnectorName)) {

							// copy incoming connections
							EList<IConnection> wrappedIncomingConnectionsList =
									new BasicEList<IConnection>();
							wrappedIncomingConnectionsList.addAll(wrappedConnector
									.getIncomingList());
							for(IConnection connection : wrappedIncomingConnectionsList) {
								elemRefConnector.getIncomingList().add(connection);
							}

							// copy outgoing connections
							EList<IConnection> wrappedOutgoingConnectionsList =
									new BasicEList<IConnection>();
							wrappedOutgoingConnectionsList.addAll(wrappedConnector
									.getOutgoingList());
							for(IConnection connection : wrappedOutgoingConnectionsList) {
								elemRefConnector.getOutgoingList().add(connection);
							}
						}
					}
				}

				copyAllLayoutData((ILayoutedModelElement)elementRef, (ILayoutedModelElement)wrapped);

				container.getContainedElementsList().add(wrapped);
			}
		});
	}

	/** Copies the layout data from source to target. */
	private static void
			copyAllLayoutData(ILayoutedModelElement source, ILayoutedModelElement target) {
		target.setLayoutData(copyAll(source.getLayoutDataList()).toArray(new ILayoutData[2]));
	}
}
