/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.objects;

import static org.lwjgl.opengl.GL11.glPopName;
import static org.lwjgl.opengl.GL11.glPushName;

import org.fortiss.tooling.graphicsGL.ui.camera.Camera;
import org.fortiss.tooling.graphicsGL.ui.picker.ModelObjectPicker;

/**
 * Base class for {@link ViewObjectBase view objects}, which become pickable by the
 * {@link ModelObjectPicker}. Sub-classes may override
 * {@link #renderSimplifiedPickableObject(Camera)} to
 * provide a simpler shape during the picking render process. The default renders the same object as
 * {@link #renderObject(Camera)}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 016B5CFB5D16A95D95E1A3DB41202FBD
 */
public abstract class PickableObjectBase extends ViewObjectBase {

	/** {@inheritDoc} */
	@Override
	public final void renderObjectForPicking(ModelObjectPicker picker, Camera c) {
		int name = picker.registerPickableObject(this);
		glPushName(name);
		renderSimplifiedPickableObject(c);
		glPopName();
	}

	/**
	 * Renders this object in a simplified version for picking. The default just renders the object
	 * as usual. Sub-classes may override to provide simpler shapes.
	 */
	protected void renderSimplifiedPickableObject(Camera c) {
		renderObject(c);
	}
}
